"""A smart propane monitor."""

import logging
import os
from tempfile import gettempdir

from pkg_resources import resource_string

__version__ = resource_string(__name__, "VERSION").decode()

logging.getLogger(__name__).addHandler(logging.NullHandler())



"""Interact with Tank Utility devices."""

import requests

from . import auth
from . import common


def get_devices(token):
    """Get devices associated with ``token``.

    :param str token:
    :rtype: :py:class:`list`
    :raises requests.exceptions.HTTPError:

    """
    response = requests.get(common.get_api_url(token=token, path="devices"))
    response.raise_for_status()
    return response.json()["devices"]


def get_device_data(token, device):
    """Get ``device`` data.

    :param str token:
    :param str device:
    :rtype: :py:class:`dict`
    :raises requests.exceptions.HTTPError:

    """
    response = requests.get(common.get_api_url(token=token, path="devices/" + device))
    response.raise_for_status()
    return response.json()


def get_data(username, password):
    """Get data for all devices associated with credentials.

    Get a new token if a cached token has expired.

    :param str username:
    :param str password:
    :rtype: :py:class:`dict`
    :returns: A mapping of device id to device data
    :raises requests.exceptions.HTTPError:

    """
    data = {}
    token = auth.get_token(username, password)
    try:
        devices = get_devices(token)
    except requests.exceptions.HTTPError as e:
        if e.response.status_code == requests.codes.unauthorized:  # pylint: disable=no-member
            devices = get_devices(auth.get_token(username, password, force=True))
        else:
            raise e
    for d in devices:
        data[d] = get_device_data(token, d)
    return data



"""API authentication."""

import contextlib
import logging
import os
import os.path
import tempfile

import requests

from . import common

TOKEN_FILE_PATH = os.path.join(tempfile.gettempdir(), "tank_utility_token.txt")
"""Name of the file to store a cached token within."""


@contextlib.contextmanager
def umask(context_umask):
    """Modify umask within a context."""
    original_umask = os.umask(context_umask)
    try:
        yield
    finally:
        os.umask(original_umask)


def _get_cached_token():
    """Get a cached API token from a temporary file.

    :rtype: :py:class:`str`
    :returns: API token or empty string

    """
    token = ""
    try:
        with open(TOKEN_FILE_PATH, "r") as f:
            token = f.read()
    except FileNotFoundError:
        logging.info("No existing file %s", TOKEN_FILE_PATH)
    return token


def _cache_token(token):
    """Cache ``token`` in a temporary file.

    :param str token:

    """
    try:
        os.remove(TOKEN_FILE_PATH)
    except FileNotFoundError:
        logging.info("No existing file %s", TOKEN_FILE_PATH)
    with umask(0o077):
        with open(TOKEN_FILE_PATH, "w") as f:
            f.write(token)
        logging.info("Cached token in %s", TOKEN_FILE_PATH)


def get_token(username, password, force=False):
    """Get an API token.

    Unfortunately, the Tank Utility API only offers basic authentication and expires each API token after 24 hours.

    :param str username:
    :param str password:
    :param bool force: (optional) Get a new token even if one is currently cached
    :rtype: :py:class:`str`
    :returns: An API token
    :raises requests.exceptions.HTTPError:

    """
    token = None if force else _get_cached_token()
    if not token:
        response = requests.get(
            common.get_api_url(path="getToken"), auth=requests.auth.HTTPBasicAuth(username, password)
        )
        response.raise_for_status()
        token = response.json()["token"]
        _cache_token(token)
    return token



"""Common functions."""

try:
    from urllib.parse import SplitResult, urlencode, urlunsplit
except ImportError:
    from urllib import urlencode
    from urlparse import SplitResult, urlunsplit

PROPANE_API_PROTOCOL = "https"
PROPANE_API_LOCATION = "data.tankutility.com"
"""The base location of the propane API."""


def get_api_url(token="", path=""):
    """Get an API URL for the given path.

    :param str token: (optional) API token
    :param str path: (optional) API path
    :rtype: :py:class:`str`
    :returns: An API URL

    """
    query = urlencode({"token": token}) if token else ""
    api_path = "/".join(["api", path]).strip("/")
    split = SplitResult(
        scheme=PROPANE_API_PROTOCOL, netloc=PROPANE_API_LOCATION, path=api_path, query=query, fragment=""
    )
    return urlunsplit(split)



"""setuptools entry point."""

from os import path
from setuptools import find_packages, setup
import codecs

HERE = path.abspath(path.dirname(__file__))

with codecs.open(path.join(HERE, "README.rst"), encoding="utf-8") as f:
    LONG_DESCRIPTION = f.read()

with codecs.open(path.join(HERE, "src", "tank_utility", "VERSION")) as version_file:
    VERSION = version_file.read().strip()

setup(
    name="tank_utility",
    version=VERSION,
    description="A smart propane tank monitor.",
    long_description=LONG_DESCRIPTION,
    author="Kris Molendyke",
    author_email="kris@k20e.com",
    url="https://git.io/k20e",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6"
    ],
    keywords="propane monitor",
    packages=find_packages("src"),
    package_dir={"": "src"},
    install_requires=[
        "requests>=2.0",
        "urllib3[secure]"
    ],
    extras_require={
        "dev": [
            "bumpversion",
            "pylint",
            "pre-commit",
            "yapf"
        ],
        "test": [
            "coverage",
            "mock",
            "responses",
            "tox"
        ],
    },
    package_data={},
    include_package_data=True,
    data_files=[],
    test_suite="tests",
    entry_points={"console_scripts": ["tank-utility = tank_utility.__main__:main"]}
)



"""The Tank Utility CLI."""

import argparse
import json
import os

from . import __version__
from . import device


def _get_parser():
    """Get a command line argument parser.

    :param list argv: A list of command line arguments.
    :rtype: :py:class:`argparse.ArgumentParser`

    """
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("--username", help="Username", default=os.environ.get("TANK_UTILITY_USERNAME", ""))
    parser.add_argument("--password", help="Password", default=os.environ.get("TANK_UTILITY_PASSWORD", ""))
    parser.add_argument("--pretty", help="Pretty print data", action="store_true")
    parser.add_argument("--verbose", help="Print debugging information", action="store_true")
    parser.add_argument("--version", action="version", version=__version__)
    return parser


def main():
    """CLI entry point."""
    args = _get_parser().parse_args()
    if args.verbose:
        import logging
        logging.getLogger("").setLevel(logging.DEBUG)
    data = device.get_data(args.username, args.password)
    if args.pretty:
        import pprint
        pprint.pprint(data)
    else:
        print(json.dumps(data))
